#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________


import copy
import os
import imp
import sys
import pyutilib.common


try:
    import runpy
    runpy_available=True
except ImportError:   #pragma:nocover
    try:
        import runpy2 as runpy
        runpy_available=True
    except ImportError: #pragma:nocover
        runpy_available=False

def import_file(filename, context=None, name=None):
    """
    Import a Python file as a module

    The filename argument can be a module name or a python script
    (.py, .pyc).  The module name will be extracted from the filename
    argument by removing any directory components along with the .py
    (or .pyc) suffix if any of these exist. If the module name already
    appears in sys.modules, the corresponding module entry will be
    returned from this dictionary. Otherwise, an attempt will be made
    import the module name extracted from the filename argument (using
    any directory information that was included).

    The name keyword can be used to always force reloading of the
    module from scratch. The module, if succesully imported, will be
    assigned the supplied name in sys.modules. This option is not
    currently supported for module names that contain one or more
    periods that indicate a submodule.

    This function returns the module object that is created.
    """

    #
    # Parse the filename to get the name of the module to be imported
    # and determine if it contains any directory information about
    # where to find the module.
    #
    dirname = os.path.dirname(filename)
    implied_dirname = None
    if dirname == '':
        dirname = None
        implied_dirname = os.path.dirname(os.path.abspath(filename))
    modulename = os.path.basename(filename)
    is_file = False
    # NB: endswith accepts tuples of strings starting in python 2.5.
    # For 2.4 compatibility we will call endswith() twice.
    if modulename.endswith('.py') or modulename.endswith('.pyc'):
        if not os.path.exists(filename):
            raise IOError("File %s does not exist" % (filename))
        if filename.endswith('.pyc'):
            filename = filename[:-1]
        modulename = modulename.rsplit('.', 1)[0]
        is_file = True

    module = None
    # If an import name is not given we will first check if the module
    # name already exists sys.modules due to a previous import and if
    # not we will try to import
    # **NOTE**: This will ignore any directory specifications that
    #           were included in the filename, meaning
    #              filename=foo/bar.py
    #           would be overlooked if someone previously imported any
    #           module 'bar' from anywhere else. As a workaround for
    #           this issue, the 'name' keyword was added to this
    #           function, which indicates that the module source
    #           should be obtained loaded from scratch with and
    #           assigned the module name specified by this keyword.
    if name is None:
        name = modulename
        if modulename in sys.modules:
            module = sys.modules[modulename]
        else:
            if dirname is not None:
                sys.path.insert(0, dirname)
            else:
                sys.path.insert(0, implied_dirname)
            try:
                module = __import__(modulename)
            except ImportError:
                raise
            finally:
                if dirname is not None:
                    sys.path.remove(dirname)
                else:
                    sys.path.remove(implied_dirname)
    else:
        pathname = None
        if is_file:
            pathname = filename
        else:
            if "." in modulename:
                raise ValueError("Unsupported module name '%s'. "
                                 "Module names cannot contain a "
                                 "period when the 'name' keyword "
                                 "is used." % modulename)
            if dirname is not None:
                # find_module will return the .py file (never .pyc)
                fp, pathname, description = imp.find_module(modulename, [dirname])
                fp.close()
            else:
                sys.path.insert(0, implied_dirname)
                try:
                    # find_module will return the .py file (never .pyc)
                    fp, pathname, description = imp.find_module(modulename)
                    fp.close()
                except ImportError:
                    raise
                finally:
                    sys.path.remove(implied_dirname)
        try:
            # Note: we are always handing load_source a .py
            #       file, but it will use the .pyc or .pyo 
            #       file if it exists
            module = imp.load_source(name, pathname)
        except:
            et,e,tb = sys.exc_info()
            import traceback
            _,line,_,txt = traceback.extract_tb(tb,2)[-1]
            import logging
            logger = logging.getLogger('pyutilib.misc')
            msg = ''
            if isinstance(e,Exception):
                msg = " raised %s:\n%s" % (et.__name__, e)
            logger.error( 'Failed to load python module "%s"\n'
                          '%s line %s ("%s")%s'
                          % ( filename, pathname, line, txt, msg) )
            raise

    #
    # Add module to the give context
    #
    if not context is None:
        context[name] = module
    return module


def run_file(filename, logfile=None, execdir=None):
    """
    Execute a Python file and optionally redirect output to a logfile.
    """
    if not runpy_available:                     #pragma:nocover
        raise pyutilib.common.ConfigurationError("Cannot apply the run_file() function because runpy is not available") 
    #
    # Open logfile
    #
    if not logfile is None:
        sys.stderr.flush()
        sys.stdout.flush()
        save_stdout = sys.stdout
        save_stderr = sys.stderr
        OUTPUT=open(logfile,"w")
        sys.stdout=OUTPUT
        sys.stderr=OUTPUT
    #
    # Add the file directory to the system path
    #
    if '/' in filename:
        tmp= "/".join((filename).split("/")[:-1])
        tmp_import = (filename).split("/")[-1]
        sys.path.append(tmp)
    elif '\\' in filename:
        tmp = "\\".join((filename).split("\\")[:-1])
        tmp_import = (filename).split("\\")[-1]
        sys.path.append(tmp)
    else:
        tmp_import = filename
    name = ".".join((tmp_import).split(".")[:-1])
    #
    # Run the module
    #
    try:
        if not execdir is None:
            tmp=os.getcwd()
            os.chdir(execdir)
            tmp_path = sys.path
            sys.path = [execdir] + sys.path
        runpy.run_module(name,None,"__main__")
        if not execdir is None:
            os.chdir(tmp)
            sys.path = tmp_path
    except Exception:          #pragma:nocover
        if not logfile is None:
            OUTPUT.close()
            sys.stdout = save_stdout
            sys.stderr = save_stderr
        raise
    #
    # Close logfile
    #
    if not logfile is None:
        OUTPUT.close()
        sys.stdout = save_stdout
        sys.stderr = save_stderr
